Package: PTT
Type: Package
Title: Pólya Tree Type Models
Version: 1.0.1
Authors@R: person("Li", "Ma", email = "mastatlab@gmail.com",
    role = c("aut", "cre"))
Description: Fits Bayesian nonparametric models based on Pólya tree
    processes, including adaptive Pólya trees, Markov adaptive Pólya trees,
    optional Pólya trees, and their conditional-density counterparts.
    Methods are described in Ma (2017) <doi:10.1214/16-BA1021>, Ma (2017)
    <doi:10.1214/17-EJS1254>, and Wong and Ma (2010)
    <doi:10.1214/09-AOS755>.
License: GPL (>= 3)
URL: https://github.com/MaStatLab/PTT
BugReports: https://github.com/MaStatLab/PTT/issues
Encoding: UTF-8
Imports: Rcpp (>= 1.0.0)
LinkingTo: Rcpp, RcppArmadillo
NeedsCompilation: yes
Config/roxygen2/version: 8.0.0
Packaged: 2026-07-22 23:27:27 UTC; li.ma
Author: Li Ma [aut, cre]
Maintainer: Li Ma <mastatlab@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-23 10:50:41 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-23 11:47:27 UTC; unix
Archs: PTT.so.dSYM
