# Generated by roxygen2: do not edit by hand

S3method(plot,part)
S3method(plot,roc)
export(apt)
export(cond.apt)
export(cond.opt)
export(opt)
export(plot.part)
export(plot.roc)
export(plot_partition)
export(plot_roc_curve)
importFrom(Rcpp,evalCpp)
useDynLib(PTT, .registration = TRUE)
